@@ -491,7 +491,7 @@ class Gr00tN1(ActionModel):
         **kwargs,
     ):
         super().__init__(server_url, server_port)
-        self.client = ExternalRobotInferenceClient(server_url, server_port)
+        self.client = ExternalRobotInferenceClient(host=server_url, port=server_port)
         self.action_keys = action_keys
 
     def sample_actions(self, inputs: dict) -> np.ndarray:
@@ -711,7 +711,7 @@ class Gr00tN1(ActionModel):
         fps: int = 30,
         speed: float = 1.0,
         cameras_keys_mapping: Dict[str, int] | None = None,
-        angle_format: Literal["degrees", "radians", "other"] = "radians",
+        unit: Literal["degrees", "rad", "other"] = "rad",
         min_angle: float | None = None,
         max_angle: float | None = None,
         **kwargs: Any,
@@ -726,11 +726,6 @@ class Gr00tN1(ActionModel):
         nb_iter = 0
         config = model_spawn_config.hf_model_config
         signal_marked_as_started = False
-        unit: Literal["degrees", "rad", "other"]
-        if angle_format != "radians":
-            unit = angle_format
-        else:
-            unit = "rad"
 
         while control_signal.is_in_loop():
             logger.debug(
@@ -841,6 +836,7 @@ class Gr00tN1(ActionModel):
                 control_signal.set_running()
                 signal_marked_as_started = True
 
+            nb_actions_too_large = 0
             for action in actions:
                 # Early stop
                 if not control_signal.is_in_loop():
@@ -848,21 +844,79 @@ class Gr00tN1(ActionModel):
                 # Send the new joint position to the robot
                 action_list = action.tolist()
                 for robot_index in range(len(robots)):
-                    if all(
-                        np.isclose(
-                            action_list[robot_index * 6 : robot_index * 6 + 6],
-                            -np.pi if unit == "rad" else -180,
+                    target_position = action_list[robot_index * 6 : robot_index * 6 + 6]
+
+                    # If the distance between the current and target position is too high, skip the action
+                    current_position = robots[robot_index].read_joints_position(
+                        unit=unit,
+                        max_value=max_angle,
+                        min_value=min_angle,
+                        source="sim",
+                    )
+                    max_transition_angles: np.ndarray
+                    if unit == "degrees":
+                        # The last joint is the gripper, which can open/close
+                        max_transition_angles = np.array([90.0] * 5 + [180.0])
+                        current_to_target_diff = np.abs(
+                            (target_position - current_position + 180) % 360 - 180
                         )
+
+                    elif unit == "rad":
+                        # The last joint is the gripper, which can open/close
+                        max_transition_angles = np.array([np.pi / 2] * 5 + [np.pi])
+                        current_to_target_diff = np.abs(
+                            (target_position - current_position + np.pi) % (2 * np.pi)
+                            - np.pi
+                        )
+                    elif (
+                        unit == "other"
+                        and max_angle is not None
+                        and min_angle is not None
                     ):
-                        logger.warning("All predicted actions are -pi. Skipping.")
-                        continue
+                        # The last joint is the gripper, which can open/close
+                        max_transition_angle = (max_angle - min_angle) / 2
+                        max_transition_angles = np.array(
+                            [max_transition_angle] * 5 + [max_angle - min_angle]
+                        )
+                        current_to_target_diff = np.abs(
+                            (target_position - current_position + max_angle)
+                            % (max_angle - min_angle)
+                            - max_transition_angle
+                        )
+                    else:
+                        raise ValueError(f"Unknown unit: {unit}")
+
+                    if np.any(current_to_target_diff > max_transition_angles):
+                        largest_diff = np.max(current_to_target_diff)
+                        largest_diff_index = np.argmax(current_to_target_diff)
+                        error_message = (
+                            f"Skipping action for robot {robot_index} because the to joint position {largest_diff_index} difference is too large: {largest_diff} > {max_transition_angles[largest_diff_index]} in units {unit}"
+                            + f"\nCurrent position: {current_position}"
+                            + f"\nTarget position: {target_position}\n"
+                            + "Possible reasons for this error:"
+                            + "\n1. Make sure you selected the *right angle unit* in the control page (angle, degrees, other)."
+                            + "\n2. Inspect your dataset joints positions to ensure they are within the expected range."
+                            + "\n3. There was an issue in the model output, please check the model training and data quality."
+                        )
+                        if nb_actions_too_large <= 20:
+                            logger.warning(error_message)
+                            nb_actions_too_large += 1
+                            continue
+                        else:
+                            control_signal.stop()
+                            raise Exception(error_message)
+                    else:
+                        logger.debug(
+                            f"Writing joint position to robot {robot_index}: {target_position}"
+                        )
 
                     robots[robot_index].write_joint_positions(
-                        angles=action_list[robot_index * 6 : robot_index * 6 + 6],
+                        angles=target_position,
                         unit=unit,
                         max_value=max_angle,
                         min_value=min_angle,
                     )
+                    nb_actions_too_large = 0
 
                 # Wait fps time
                 elapsed_time = time.perf_counter() - start_time
