@@ -63,6 +63,9 @@ class Configuration(BaseModel):
     # HF token
     HF_TOKEN_VALID: bool = False
 
+    # Private mode
+    DEFAULT_HF_PRIVATE_MODE: bool = False
+
     # These fields will be set after loading the user config
     DEFAULT_DATASET_NAME: str = "example_dataset"
     DEFAULT_FREQ: int = 30
