@@ -211,14 +211,6 @@ class BaseManipulator(BaseRobot):
         if axis is None:
             axis = axis_robot.new_position()
 
-        # When creating a new robot, you should add default values for these
-        # These values depends on the hardware
-        assert self.CALIBRATION_POSITION is not None, (
-            "CALIBRATION_POSITION must be defined in the class"
-        )
-        assert self.RESOLUTION is not None, "RESOLUTION must be defined in the class"
-        assert self.SERVO_IDS is not None, "SERVO_IDS must be defined in the class"
-
         if serial_id is not None:
             self.SERIAL_ID = serial_id
         else:
@@ -249,6 +241,23 @@ class BaseManipulator(BaseRobot):
 
         self.actuated_joints = actuated_joints
 
+        # Infer SERVO_IDS and CALIBRATION_POSITION from the actuated joints if not set
+        if not hasattr(self, "SERVO_IDS"):
+            self.SERVO_IDS = list(range(1, len(self.actuated_joints) + 1))
+            logger.warning(
+                f"{self.__class__.__name__}.SERVO_IDS not set, using default: {self.SERVO_IDS}"
+            )
+        if not hasattr(self, "CALIBRATION_POSITION"):
+            self.CALIBRATION_POSITION = [0] * len(self.SERVO_IDS)
+            logger.warning(
+                f"{self.__class__.__name__}.CALIBRATION_POSITION not set, using default: {self.CALIBRATION_POSITION}"
+            )
+        if not hasattr(self, "SLEEP_POSITION"):
+            self.SLEEP_POSITION = [0] * len(self.SERVO_IDS)
+            logger.warning(
+                f"{self.__class__.__name__}.SLEEP_POSITION not set, using default: {self.SLEEP_POSITION}"
+            )
+
         self.sim.set_joints_states(
             robot_id=self.p_robot_id,
             joint_indices=self.actuated_joints,
@@ -738,14 +747,14 @@ class BaseManipulator(BaseRobot):
                 # Convert from motor units to radians
                 output_position_rad = self._units_vec_to_radians(output_position)
                 # Normalize the angles to [min_value, max_value]
-
                 output_position = min_value + (max_value - min_value) * (
-                    (output_position_rad + np.pi) / (2 * np.pi)
-                )
+                    output_position_rad + np.pi
+                ) / (2 * np.pi)
+
             elif source_unit == "rad":
                 output_position = min_value + (max_value - min_value) * (
-                    (output_position + np.pi) / (2 * np.pi)
-                )
+                    output_position + np.pi
+                ) / (2 * np.pi)
         else:
             raise ValueError(
                 f"Invalid unit: {unit}. Must be one of ['rad', 'motor_units', 'degrees']"
